<?php

namespace FiloBlu\OrderFlowRectifier\Observer;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;

/**
 * Observer on Adyen notification to get cc_type value and update it in table sales_order_payment if missing
 * This observer needs the adyen module to be patched at least with adyen_notification_events_4303_update.patch
 */

class UpdateCCType implements ObserverInterface
{
    const SCHEDULED_FIX_CCTYPE_ENABLED_CONFIG_PATH = 'order_flow_rectifier/general/scheduled_cctype_check_enabled';
    const PAYMENT_METHOD_ADYEN_CC = 'adyen_cc';
    const CC_TYPE_MAP = array(
        "amex" => "AE",
        "visa" => "VI",
        "mc" => "MC",
        "discover" => "DI",
        "jcb" => "JCB",
        "diners" => "DN",
        "maestro" => "MI",
        "cup" => "UN",
        "bcmc" => "BCMC",
        "hipercard" => "HIPERCARD",
        "elo" => "ELO",
        "troy" => "TROY",
        "dankort" => "DANKORT"
    );

    /**
     * @var ScopeConfigInterface
     */
    protected $config;

    public function __construct(
        ScopeConfigInterface $config
    )
    {
        $this->config = $config;
    }

    /**
     * @param Observer $observer
     */
    public function execute(Observer $observer)
    {
        if ($this->config->isSetFlag(self::SCHEDULED_FIX_CCTYPE_ENABLED_CONFIG_PATH))
        {
            $adyenPaymentMethod = $observer->getEvent()->getPaymentMethod();
            $order = $observer->getEvent()->getOrder();
            $orderPayment = $order->getPayment();

            if($orderPayment->getMethod() == self::PAYMENT_METHOD_ADYEN_CC && is_null($orderPayment->getCcType()))
            {
                if(array_key_exists($adyenPaymentMethod, self::CC_TYPE_MAP))
                {
                    $orderPayment->setCcType(self::CC_TYPE_MAP[$adyenPaymentMethod]);
                }
            }
        }

        return TRUE;
    }
}